/***************************************************************************
 *  file                 :  gui_interaction.c                              *
 *  copyright            : (C) 2001-2005 by Dirk Nolting                   *
 *  email                : nolting@uni-duesseldorf.de                      *
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#define _GNU_SOURCE

#include "support.h"
#include "gui_interaction.h"
#include "global.h"
#include "ipc.h"
#include "ddm.h"
#include "callbacks.h"
#include "element.h"

#include <gtk/gtk.h>
#include <signal.h>
#include <math.h>
#include <stdio.h>
#include <string.h>

static GtkListStore *list_store;
static GtkWidget *status_bar;
static guint context, last_status_message=-1;
static int charge = DEFAULT_CHARGE, digits = USE_DIGITS;
static double minint=MIN_INT;


/**
 * Display an error dialog: no formula given
 */
void display_error_dlg( const char *caption, const char *text)
{
  GtkWidget *dialog, *mainwin;

  mainwin = get_mainwindow();
  dialog = gtk_message_dialog_new (GTK_WINDOW(mainwin),
				   GTK_DIALOG_DESTROY_WITH_PARENT,
				   GTK_MESSAGE_ERROR,
				   GTK_BUTTONS_CLOSE,
				   caption);
  gtk_message_dialog_format_secondary_text( GTK_MESSAGE_DIALOG(dialog), text);
  gtk_dialog_run (GTK_DIALOG (dialog));
  gtk_widget_destroy (dialog);
}

/**
 * Clears the display from previous results
 */
void clear_display()
{
  GtkWidget *mainwin, *dummy;

  mainwin = get_mainwindow();

  dummy = lookup_widget( mainwin, "formel_input" );
  gtk_entry_set_text( GTK_ENTRY( dummy ), "");
  gtk_widget_grab_focus( dummy );

  gtk_list_store_clear( list_store );
  generate_display_data(NULL);

  clear_status_bar();
  last_status_message = gtk_statusbar_push( GTK_STATUSBAR(status_bar), 
					    context,_("Ready."));
}


/**
 * Read the license file and return it in a single string
 */
char *get_license()
{
  char *license=NULL, *line=NULL;
  FILE *file;
  int length=1, line_size=0, bytes=0;

  file = open_file( "COPYING" );
  license = strdup( "" );

  while( (line_size=getline( &line, &bytes, file)) > -1 )
    {
      length += line_size;

      license = (char *)realloc( license, length*sizeof(char) );
      license = strcat( license, line );

      free( line );
      line = NULL;
    }

  fclose( file );

  return license;
}


/**
 *set charges for display
 */
void set_charge(int charges){
  charge = charges;
}


/**
 *get current uses charges
 */
int get_charge(){
  return charge;
}

/**
 *set digits for display
 */
void set_digits(int dig){
    digits = dig;
}


/**
 *get digits used for display
 */
int get_digits(){
    return digits;
}


/** 
 *set minimum intensity which will be displayed
 */
void set_minint(double intensity)
 {
   minint=intensity;
 }


/** 
 *get minimum intensity which will be displayed
 */
double get_minint(void)
 {
   return minint;
 }

  

/**
 * Retrieve formula input string
 */
G_CONST_RETURN gchar *get_formula_input(){
  GtkWidget *mainwindow, *input;
  G_CONST_RETURN gchar *text;

  mainwindow = get_mainwindow();
  input =  lookup_widget( GTK_WIDGET( mainwindow ), "formel_input" );
  text = gtk_entry_get_text( (GtkEntry *)input );

  return text;
}


/**
 *Remove previous status bar message
 */
void clear_status_bar(){
  if( last_status_message > -1 )
    gtk_statusbar_remove( GTK_STATUSBAR(status_bar), 
			  context, 
			  last_status_message );
}


/**
 * Generate three columns for displaying the results
 */
void init_show_result(GtkWidget *main)
{
  GtkWidget *tree;
  GtkCellRenderer *renderer;
  GtkTreeViewColumn *column;

  tree = lookup_widget( main, "show_result" );
  status_bar = lookup_widget( main, "statusbar" );

  list_store = gtk_list_store_new( 
				  COL_NUMBER,
				  G_TYPE_STRING,
				  G_TYPE_STRING,
				  G_TYPE_STRING
				 );

  renderer = gtk_cell_renderer_text_new ();
  column = gtk_tree_view_column_new_with_attributes (
						     _("Mass"),
						     renderer,
						     "text",
						     COL_MASS,
						     NULL);
  gtk_tree_view_append_column (GTK_TREE_VIEW (tree), column);

  column = gtk_tree_view_column_new_with_attributes (
						     _("rel. Intensity"),
						     renderer,
						     "text",
						     COL_INTENSITY,
						     NULL);

  gtk_tree_view_append_column (GTK_TREE_VIEW (tree), column);

  column = gtk_tree_view_column_new_with_attributes (
						     _("Probability"),
						     renderer,
						     "text",
						     COL_PROBABILITY,
						     NULL);

  gtk_tree_view_append_column (GTK_TREE_VIEW (tree), column);

  gtk_tree_view_set_model( GTK_TREE_VIEW( tree ), GTK_TREE_MODEL(list_store) );

  context = gtk_statusbar_get_context_id( GTK_STATUSBAR(status_bar), 
					  "allgemein" );
  last_status_message = gtk_statusbar_push( GTK_STATUSBAR(status_bar), 
					    context,_("Ready."));

}


/**
 * Display the results
 */
void display_result(){
  isotope *d, *display=NULL;
  double maxp=0,relint=0,sump=0;
  int permutationen=0;
  char mass[255], inten[255], prob[255], perm[255], *formatm, *formati;
  gboolean fast, show_all;

  GtkTreeIter iter;

  gtk_list_store_clear( list_store );
  generate_display_data( peaks );

  formatm = get_massformat();
  formati = get_intformat();

  maxp = get_maxp();
  permutationen = get_permutations();
  show_all = is_show_all_active();
  display = get_display_data();

  for(d=display;d;d=d->next)
    {
	if( (relint=((d->p/maxp)*100)) > minint || show_all ){
	    sprintf( mass,formatm, d->mass);
	    sprintf( inten,formati, relint);
	    sprintf( prob,"%e", d->p);
	    gtk_list_store_append( list_store, &iter );
	    gtk_list_store_set( list_store, &iter,
				COL_MASS,mass,
				COL_INTENSITY,inten,
				COL_PROBABILITY,prob,
				-1);
	}
    }

  fast = do_fastcalc();
  if(!( fast )){
    sprintf(perm,_("Number of  permutations: %i"),permutationen);
  }
  else
    {
      sump = get_sump();
      sump = (rint(sump*10000)/100); 
      sprintf( perm, _("Covered Intensity: %2.2f%%"),sump);
    }
  clear_status_bar();
  last_status_message = gtk_statusbar_push( GTK_STATUSBAR( status_bar ), 
					    context, 
					    perm);
}

void save_result(gchar *path)
{
  isotope *d, *display=NULL;
  double maxp=0,relint=0;
  int permutationen=0, strlength, charge;
  char *formatm, *formati, *formatstr, *formula;
  FILE *file;
  gboolean show_all;

  if( !(display = get_display_data()) )
      return;

  if( !(file = fopen( path, "w")) )
      return;

  formatm = get_massformat();
  formati = get_intformat();
  maxp = get_maxp();
  permutationen = get_permutations();
  charge = get_charge();
  show_all = is_show_all_active();

  strlength = strlen( formatm );
  strlength += strlen( formati );
  strlength += 6; /* %e + 2*\t + \n + \0 */

  formatstr = (char *)malloc( strlength * sizeof( char ) );
  formatstr = strcpy( formatstr, formatm);
  formatstr = strcat( formatstr, "\t");
  formatstr = strcat( formatstr, formati);
  formatstr = strcat( formatstr, "\t");
  formatstr = strcat( formatstr, "%e\n");

  formula = (char *)get_formula_input();

  switch( get_formulatype() )
  {
      case SUM:
	  fprintf(file, _("Sum formula: %s\n"), formula);
	  break;
      case PEPTIDESEQUENCE:
	  fprintf( file, _("Peptide sequence: %s\n"), formula);
	  break;
  }

  if( show_all )
      fprintf( file, _("Complete list of peaks \n"));
  else
      fprintf( file, _("Minimum intensity displayed: %f\n"), minint);
  fprintf( file, _("Assumed charges: %i\n\n"), charge);

  for(d=display;d;d=d->next)
    {
	if( (relint=((d->p/maxp)*100)) > minint || show_all )
	    fprintf( file, formatstr, d->mass, relint, d->p);
    }

  fclose( file );
  free( formatstr );

    clear_status_bar();
    last_status_message = gtk_statusbar_push( GTK_STATUSBAR( status_bar ), 
					     context, 
					     _("saved."));

}
